@@ -7,7 +7,7 @@
 #' dynamic average treatment effects). `emfx` is a light wrapper around the
 #' \code{\link[marginaleffects]{slopes}} function from the **marginaleffects**
 #' package.
-#' 
+#'
 #' @param object An `etwfe` model object.
 #' @param type Character. The desired type of post-estimation aggregation.
 #' @param by_xvar Logical. Should the results account for heterogeneous
@@ -30,7 +30,7 @@
 #' @param collapse Logical. An alias for `compress` (only used for backwards
 #'   compatability and ignored if both arguments are provided). The behaviour is
 #'   identical, but it will trigger a message nudging users to rather use the
-#'   `compress` argument. 
+#'   `compress` argument.
 #' @param predict Character. The type (scale) of prediction used to compute the
 #'   marginal effects. If `"response"` (the default), then the output is at the
 #'   level of the response variable, i.e. it is the expected predictor
@@ -58,9 +58,24 @@
 #'   used in the `emfx` context. The upside is a potentially dramatic reduction
 #'   in the size of the return object. Consequently, we may change the default
 #'   to `TRUE` in a future version of **etwfe**.
+#' @param window Numeric of length 1 or 2. Limits the temporal window of
+#'   consideration around treatment.
+#'
+#'   - NULL (default): Include all available periods.
+#'   - Length 1: Truncate to a symmetric window around the treatment event.
+#'     E.g., `window = 2` will truncate to two pre-treatment periods and two
+#'     post-treatment periods.
+#'   - Length 2: Asymmetric window, where the first number gives the maximum
+#'     number of pre-treatment periods and the second number gives the maximum
+#'     number of post-treatment periods. E.g., `window = c(5, 2)` will truncate
+#'     to five pre-treatment periods and two post-treatment periods.
+#'
+#' Note that the pre-treatment truncation is only ever binding in cases where
+#' the "never" treated group is used as a control, i.e.,
+#' `etwfe(..., cgroup = "never")` in the original call.
 #' @param ... Additional arguments passed to
-#'   [`marginaleffects::slopes`]. For example, you can pass `vcov =
-#'   FALSE` to dramatically speed up estimation times of the main marginal
+#'   [`marginaleffects::slopes`]. For example, you can pass `vcov = FALSE`
+#'   to dramatically speed up estimation times of the main marginal
 #'   effects (but at the cost of not getting any information about standard
 #'   errors; see Performance tips below). Another potentially useful
 #'   application is testing whether heterogeneous treatment effects (i.e., the
@@ -82,10 +97,10 @@
 #'   - `s.value`
 #'   - `conf.low`
 #'   - `conf.high`
-#' @references 
+#' @references
 #' Wong, Jeffrey _et al._ (2021). \cite{You Only Compress Once: Optimal Data
 #' Compression for Estimating Linear Models}. Working paper (version: March 16,
-#' 2021). Available: 
+#' 2021). Available:
 #' https://doi.org/10.48550/arXiv.2102.11297
 #' @seealso [marginaleffects::slopes] which does the heavily lifting behind the
 #' scenes. [`etwfe`] is the companion estimating function that should be run
@@ -97,33 +112,26 @@
 #' @importFrom marginaleffects slopes
 #' @export
 emfx = function(
-    object,
-    type = c("simple", "group", "calendar", "event"),
-    by_xvar = "auto",
-    compress = "auto",
-    collapse = compress,
-    predict = c("response", "link"),
-    post_only = TRUE,
-    lean = FALSE,
-    ...
+  object,
+  type = c("simple", "group", "calendar", "event"),
+  by_xvar = "auto",
+  compress = "auto",
+  collapse = compress,
+  predict = c("response", "link"),
+  post_only = TRUE,
+  window = NULL,
+  lean = FALSE,
+  ...
 ) {
-  
   dots = list(...)
-  
-  ## FIXME: uncomment when marginaleffects 0.25.0 is released and remove
-  ## workaround at the boom of this function
-  ## https://github.com/vincentarelbundock/marginaleffects/pull/1296
-  # if (isTRUE(lean)) {
-  #   oldlean = getOption("marginaleffects_lean")
-  #   options(marginaleffects_lean = TRUE)
-  #   on.exit(options(marginaleffects_lean = oldlean))
-  # }
 
   .Dtreat = NULL
   type = match.arg(type)
   predict = match.arg(predict)
   etwfe_attr = attr(object, "etwfe")
   etwfe_attr[["type"]] = type
+  yvar = etwfe_attr[["yvar"]]
+  ctrls = etwfe_attr[["ctrls"]]
   gvar = etwfe_attr[["gvar"]]
   tvar = etwfe_attr[["tvar"]]
   ivar = etwfe_attr[["ivar"]]
@@ -131,26 +139,35 @@ emfx = function(
   gref = etwfe_attr[["gref"]]
   tref = etwfe_attr[["tref"]]
   cgroup = etwfe_attr[["cgroup"]]
-  if (!by_xvar %in% c("auto", TRUE, FALSE)) stop("\"by_xvar\" has to be \"auto\", TRUE, or FALSE.")
-  if (!collapse %in% c("auto", TRUE, FALSE)) stop("\"collapse\" has to be \"auto\", TRUE, or FALSE.")
-  
+  fe = etwfe_attr[["fe"]]
+  if (!by_xvar %in% c("auto", TRUE, FALSE)) {
+    stop("\"by_xvar\" has to be \"auto\", TRUE, or FALSE.")
+  }
+  if (!collapse %in% c("auto", TRUE, FALSE)) {
+    stop("\"collapse\" has to be \"auto\", TRUE, or FALSE.")
+  }
+
   # sanity check
   if (isTRUE(by_xvar)) {
-    if(is.null(xvar)){
+    if (is.null(xvar)) {
       warning(
         "An \"xvar\" attribute was not found as part of the supplied model object. ",
         "(Did your original `etwfe()` call include a valid `xvar = ...` argument?)",
         "Average margins are reported instead."
-        )
+      )
       by_xvar = FALSE
-      }
+    }
+  }
+
+  if (by_xvar == "auto") {
+    by_xvar = !is.null(xvar)
   }
 
-  if (by_xvar=="auto") by_xvar = !is.null(xvar)
-  
   dat = as.data.table(eval(object$call$data, object$call_env))
-  if ("group" %in% names(dat)) dat[["group"]] = NULL
-  
+  if ("group" %in% names(dat)) {
+    dat[["group"]] = NULL
+  }
+
   # check compress argument
   nrows = NULL
   if (compress == "auto" && collapse != compress) {
@@ -169,7 +186,7 @@ emfx = function(
       compress = FALSE
     }
   }
-  
+
   if (cgroup == "never") {
     dat = dat[dat[[gvar]] != gref, , drop = FALSE] # Drop never treated reference group
     if (type != "event") {
@@ -177,76 +194,135 @@ emfx = function(
       dat = dat[dat[[".Dtreat"]], , drop = FALSE]
       dat = dat[dat[[tvar]] >= dat[[gvar]], , drop = FALSE]
     }
-  } else if (type=="event" & isFALSE(post_only)) {
+  } else if (type == "event" & isFALSE(post_only)) {
     dat = dat[dat[[gvar]] != gref, , drop = FALSE]
   } else if (".Dtreat" %in% names(dat)) {
     dat = dat[dat[[".Dtreat"]], , drop = FALSE]
   }
-  
+
+  if (!is.null(window)) {
+    if (length(window) == 1) {
+      dat = dat[
+        dat[[tvar]] >= (dat[[gvar]] - window) &
+          dat[[tvar]] <= (dat[[gvar]] + window),
+        ,
+        drop = FALSE
+      ]
+    } else if (length(window) == 2) {
+      dat = dat[
+        dat[[tvar]] >= (dat[[gvar]] - window[1]) &
+          dat[[tvar]] <= (dat[[gvar]] + window[2]),
+        ,
+        drop = FALSE
+      ]
+    } else {
+      stop("Invalid `window` argument. Must be of length 1 or 2.")
+    }
+  }
+
   # define formulas and calculate weights
-  if(compress & is.null(ivar)){
-    if(by_xvar){
+  if (compress & is.null(ivar)) {
+    if (by_xvar) {
       dat_weights = dat[(.Dtreat)][, .N, by = c(gvar, tvar, xvar)]
     } else {
       dat_weights = dat[(.Dtreat)][, .N, by = c(gvar, tvar)]
     }
-    
-   if (!is.null(nrows) && nrows > 5e5) warning(
-    "\nNote: Dataset larger than 500k rows detected. The data will be ",
-    "compressed by period-cohort groups to reduce estimation times. ", 
-    "However, this shortcut can reduce the accuracy of the reported ",
-    "marginal effects. ",
-    "To override this default behaviour, specify: ",
-    "`emfx(..., compress = FALSE)`\n"
-    ) 
-    
+
+    if (!is.null(nrows) && nrows > 5e5) {
+      warning(
+        "\nNote: Dataset larger than 500k rows detected. The data will be ",
+        "compressed by period-cohort groups to reduce estimation times. ",
+        "However, this shortcut can reduce the accuracy of the reported ",
+        "marginal effects. ",
+        "To override this default behaviour, specify: ",
+        "`emfx(..., compress = FALSE)`\n"
+      )
+    }
+
+    # id factor / character columns for dropping, since we can't compute means
+    # on these (https://github.com/Rdatatable/data.table/issues/7261)
+    scols = sapply(dat, function(x) is.factor(x) || is.character(x))
+    scols[c(gvar, tvar, xvar, ".Dtreat")] = TRUE
     # compress the data
-    dat = dat[(.Dtreat)][, lapply(.SD, mean), by = c(gvar, tvar, xvar, ".Dtreat")] # compress data
+    dat = dat[(.Dtreat)][,
+      lapply(.SD, mean),
+      by = c(gvar, tvar, xvar, ".Dtreat"),
+      .SDcols = !scols
+    ]
     dat = setDT(dat)[, merge(.SD, dat_weights, all.x = TRUE)] # add weights
-    
-    
   } else if (compress & !is.null(ivar)) {
-    warning("\"ivar\" is not NULL. Marginal effects are calculated without collapsing.")
+    warning(
+      "\"ivar\" is not NULL. Marginal effects are calculated without collapsing."
+    )
     dat$N = 1L
-    
   } else {
     dat$N = 1L
   }
-  
-  # compress the data 
-  if (type=="simple") {
-      by_var = ".Dtreat"
-  } else if (type=="group") {
+
+  # compress the data
+  if (type == "simple") {
+    by_var = ".Dtreat"
+  } else if (type == "group") {
     by_var = gvar
-  } else if (type=="calendar") {
+  } else if (type == "calendar") {
     by_var = tvar
-  } else if (type=="event") {
+  } else if (type == "event") {
     dat[["event"]] = dat[[tvar]] - dat[[gvar]]
     by_var = "event"
   }
-  
-  if (by_xvar) by_var = c(by_var, xvar)
 
-  mfx = slopes(
-    object,
-    newdata = dat,   
+  if (by_xvar) {
+    by_var = c(by_var, xvar)
+  }
+
+  # catch for non-linear models
+  if (!is.null(dots)) {
+    if (
+      !is.null(object$family) &&
+        object$family$family != "gaussian" &&
+        fe != "none"
+    ) {
+      if (is.null(dots$vcov) || !isFALSE(dots$vcov)) {
+        dots$vcov = FALSE
+        warning(
+          'Cannot estimate standard errors for non-Gaussian model families in the presence of fixed effects.\n',
+          'To calculate standard errors, re-estimate the original model with `etwfe(..., fe = "none")`.\n'
+        )
+      }
+    }
+  }
+
+  if (isTRUE(lean)) {
+    old_lean = getOption("marginaleffects_lean", default = FALSE)
+    options(marginaleffects_lean = TRUE)
+    on.exit(options(marginaleffects_lean = old_lean))
+  }
+
+  ## Note: Rather use do.call so can pass adjusted dots if vcov is overriden.
+  # mfx = slopes(
+  #   object,
+  #   newdata = dat,
+  #   wts = "N",
+  #   variables = ".Dtreat",
+  #   by = by_var,
+  #   type = predict,
+  #   ...
+  # )
+  args_list = list(
+    model = object,
+    newdata = dat,
     wts = "N",
     variables = ".Dtreat",
     by = by_var,
-    type = predict,
-    ...
+    type = predict
   )
-  
-  ## FIXME: remove when marginaleffects 0.25.0 is released
-  ## https://github.com/vincentarelbundock/marginaleffects/pull/1296
-  if (isTRUE(lean)) {
-    atts = names(attributes(mfx))
-    for (a in setdiff(atts, c("names", "row.names", "class", "by", "conf_level", "lean"))) attr(mfx, a) = NULL
+  if (!is.null(dots)) {
+    args_list = utils::modifyList(args_list, dots)
   }
-  
+  mfx = do.call("slopes", args = args_list)
+
   class(mfx) = c("emfx", class(mfx))
   attr(mfx, "etwfe") = etwfe_attr
-  
+
   return(mfx)
 }
-
